
#include <Wire.h>
#include "MP1090.h"

static word MP1090_TUNE_VALUE = 0x8800;
static word MP1090_VOLUME_VALUE = 0xC027;

MP1090::MP1090()
{

}

// PUBLIC FUNCTIONS

// void MP1090::Init(byte PWR_PIN)
//  
//  :
//           byte PWR_PIN -   /  Arduino (0, 1, 2...),
//                              PWR_ON  MP1090
void MP1090::Init(byte PWR_PIN)
{
  pinMode(PWR_PIN, OUTPUT);
  digitalWrite(PWR_PIN, HIGH);
  delay (300);
  digitalWrite(PWR_PIN, LOW);
  delay (500);
}
//------------------------------------------------------------------------------
// boolean MP1090::SetStation(long StationFrequency)
//  
//  :
//           long StationFrequency -    
//  :
//           true -  ;
//           false -   .
boolean MP1090::SetStation(long StationFrequency)
{
  word StationCode = GetStationCode(StationFrequency);
  
  if (StationCode != false)
  {
    MP1090_TUNE_VALUE = 0x8800 | StationCode;
    WriteRegister(MP1090_TUNE, MP1090_TUNE_VALUE);
    return true;
  }
  else
    return false;
}
//------------------------------------------------------------------------------
// void MP1090::SetVolume(byte Volume)
//  
//  :
//           byte Volume -  :  0  15.
//               7.
void MP1090::SetVolume(byte Volume)
{  
   if (Volume <= VOLUME_MASK)
   {
     MP1090_VOLUME_VALUE &= ~VOLUME_MASK;
     MP1090_VOLUME_VALUE |= Volume;  
   
     WriteRegister(MP1090_VOLUME, MP1090_VOLUME_VALUE);
   }
}
//------------------------------------------------------------------------------
// void MP1090::SetBass(byte Bass)
//   
//  :
//           byte Bass -   :  0  3.
//               0.
void MP1090::SetBass(byte Bass)
{
   if (Bass <= 3)
   {
     MP1090_VOLUME_VALUE &= ~BASS_MASK;
     MP1090_VOLUME_VALUE |= (Bass << 8);
     
     WriteRegister(MP1090_VOLUME, MP1090_VOLUME_VALUE);
   }
}
//------------------------------------------------------------------------------
// long MP1090::GetStation (void)
//     
//  :     
long MP1090::GetStation (void)
{
   return ((MP1090_TUNE_VALUE & STATION_MASK)*50 + 64000);
}
//------------------------------------------------------------------------------
// byte MP1090::GetVolume (void)
//   
//  :  
byte MP1090::GetVolume (void)
{
   return (MP1090_VOLUME_VALUE & VOLUME_MASK);
}
//------------------------------------------------------------------------------
// byte MP1090::GetBass (void)
//     
//  :    
byte MP1090::GetBass (void)
{
   return ((highByte (MP1090_VOLUME_VALUE)) & 0x3);
}
//------------------------------------------------------------------------------
// PRIVATE FUNCTIONS
//------------------------------------------------------------------------------
void MP1090::WriteRegister(byte Register, word Value)
{  
   Wire.beginTransmission(MP1090_ADDRESS);
   Wire.write (Register);
   Wire.write (highByte (Value)); 
   Wire.write (lowByte (Value)); 
   Wire.endTransmission();  
}
//------------------------------------------------------------------------------
word MP1090::GetStationCode(long StationFrequency) // kHz
{  
   if ((StationFrequency >= 64000) && (StationFrequency <= 110000))
      return (word ((StationFrequency - 64000) / 50));

   else
       return false;
}

MP1090 MP1090_RADIO = MP1090 ();
