
#include <Wire.h>
#include "MP1092.h"


MP1092::MP1092 ()
{

}

// PUBLIC FUNCTIONS
// void MP1092::SetPeriod (int MP1092_address, byte Channel, long Period)
//   
//  :
//           int MP1092_address -  MP1092   I2C
//           (  R1, R3, R5;   0x60);
//           byte Channel -   ;   0  1;
//           long Period -    
void MP1092::SetPeriod (int MP1092_address, byte Channel, long Period) // ms
{
   byte Register;
   byte PeriodCode;

   if (Channel == 0)
      Register = MP1092_PSC0;

   else if (Channel == 1)
      Register = MP1092_PSC1;

   if (Period >= 1682)
      Period = 255;
   
   else if (Period <= 12)
      Period = 0;

   else
      Period = ((Period * 152) / 1000) - 1;

   PeriodCode =(byte) Period;
  
   WriteData(MP1092_address, Register, &PeriodCode, 1);
}
//------------------------------------------------------------------------------
// void MP1092::SetDutyCycle (int MP1092_address, byte Channel, float DutyCycle)
//       
//  :
//           int MP1092_address -  MP1092   I2C
//           (  R1, R3, R5;   0x60);
//           byte Channel -   ;   0  1;
//           float DutyCycle -        %,
//            :  0  100%
void MP1092::SetDutyCycle (int MP1092_address, byte Channel, float DutyCycle) //%
{
   byte Register;
   byte DutyCycleCode;

   if (Channel == 0)
      Register = MP1092_PWM0;

   else if (Channel == 1)
      Register = MP1092_PWM1;

   if (DutyCycle >= 100)    
      DutyCycleCode = 255;

   else if ((DutyCycle >= 0) && (DutyCycle < 100))
      DutyCycle = DutyCycle * 256 / 100;

   DutyCycleCode = (byte) DutyCycle;
     
   WriteData(MP1092_address, Register, &DutyCycleCode, 1);
}
//------------------------------------------------------------------------------
// void MP1092::LEDConfig (int MP1092_address, byte LEDNum, byte Mode)
//   
//  :
//           int MP1092_address -  MP1092   I2C
//           (  R1, R3, R5;   0x60);
//           byte LEDNum -    MP1092:  0  15;
//           byte Mode -   : 0, 1, 2  3
//           0 -    ;
//           1 -    ;
//           2 -      _0;
//           3 -      _1;
void MP1092::LEDConfig (int MP1092_address, byte LEDNum, byte Mode)// LEDNum 0...15; Mode 0...3
{
   byte LED_Register;
   byte RegValue;

   if (LEDNum <= 3)
     LED_Register = MP1092_LS0;
   
   else if ((LEDNum > 3) && (LEDNum <= 7))
     {
       LED_Register = MP1092_LS1;
       LEDNum -= 4;
     }
     
   else if ((LEDNum > 7) && (LEDNum <= 11))
     {
       LED_Register = MP1092_LS2;
       LEDNum -= 8;
     }
     
   else if ((LEDNum > 11) && (LEDNum <= 15))
     {
       LED_Register = MP1092_LS3;
       LEDNum -= 12;
     }
     
   RegValue = (byte) ReadData(MP1092_address, LED_Register);
   
   RegValue &= ~(0x3 << (LEDNum * 2));
   
   RegValue |= Mode << (LEDNum * 2);
  
   WriteData(MP1092_address, LED_Register, &RegValue, 1);
}
//------------------------------------------------------------------------------
// byte MP1092::ReadInputState (int MP1092_address, byte InputReg)
//    0...15
//  :
//           int MP1092_address -  MP1092   I2C
//           (  R1, R3, R5;   0x60);
//           byte InputReg -    ;   0  1;
//            0    0...7;     
//            1    8...15;
byte MP1092::ReadInputState (int MP1092_address, byte InputReg) // InputReg 0, 1
{
   byte Register;     

   if (InputReg == 0)
      Register = MP1092_INPUT0;
      
   if (InputReg == 1)
      Register = MP1092_INPUT1;
      
   return (byte) ReadData(MP1092_address, Register);
}
//------------------------------------------------------------------------------
// PRIVATE FUNCTIONS
void MP1092::WriteData(int MP1092_address, byte StartRegister, byte *DataPtr, byte NumOfBytes)
{  

   if (NumOfBytes > 1)
      StartRegister |= MP1092_AUTO_INCR;

   Wire.beginTransmission(MP1092_address);
   Wire.write (StartRegister);
   
   do
     {
        Wire.write (*DataPtr);
        *DataPtr++;
        NumOfBytes--;
     }
    
    while (NumOfBytes > 0);

    Wire.endTransmission();  
}
//------------------------------------------------------------------------------
char MP1092::ReadData(int MP1092_address, byte Register)
{  
    Wire.beginTransmission(MP1092_address);
    Wire.write (Register);
    Wire.endTransmission();
  
    Wire.requestFrom (MP1092_address, 1);
    return Wire.read();
}

MP1092 LED_DRIVER = MP1092 ();
